/*
Copyright (c) 2025 Mario Stöckl (mstoeck3@hs-mittweida.de). 

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <regex.h>

#define BUF 1024
#define DEBUG true
#define EXPECTED_PARAMS 2


struct datetime_t {
    int year;
    int month;
    int day;
    int hour;
    int minute;
    int second;
};

struct errorLogEntry_t {
    struct datetime_t datetime;
    char level;
    int process_id;
    int state_id;
    char *connection_id_message;
};

// https://nginx.org/en/docs/http/ngx_http_log_module.html
struct access_log_entry_t {
    char *remote_addr;
    char *remote_user;
    struct datetime_t time_local;
    char *request;
    unsigned int status;
    unsigned int body_bytes_sent;
    char *http_referer;
    char *http_user_agent;
};

void debugmsg(char *msg) {
    if(DEBUG==true){
        printf("DEBUG: %s\n", msg);
    }
}

int checkparams(int *argc, char *argv[]) {
    if(*argc != EXPECTED_PARAMS) {
        printf("Dateipfad anageben! Syntax: ./main <absoluter Dateipfad>\n");
        exit(-1);
    };
};

struct access_log_entry_t parseLogline(char *accessLogLine) {
    struct access_log_entry_t logEntry;
    //logEntry.remote_addr = parse_remote_address(accessLogLine);
    return logEntry;
};

//char* parse_remote_address(char *accessLogLine) {
//    return 1;
//}

unsigned int count_lines(FILE *filepointer) {
    rewind(filepointer);
    unsigned int lines = 0;
    char c;
    for(c=getc(filepointer); c != EOF; c = getc(filepointer)) {
        if(c=='\n'){
            lines += 1;
        }
    }
    char msg[BUF];
    sprintf(msg, "Detected %d Lines in filestream\n", lines);
    debugmsg(msg);
    return lines;
}

int main(int argc, char *argv[]) {
    debugmsg("START OF PROGRAM");
    int i = 0;
    while(i<argc){
        char msg[BUF];
        sprintf(msg,"Argument %d: %s, Speicheradresse: %p\n", i, argv[i], &argv[i]);
        debugmsg(msg);
        i++;
    }
    checkparams(&argc,  argv);
    const char *filepath = argv[1];
    FILE *fp = fopen(filepath, "r");
    if(fp == NULL) {
        perror("fopen");
        printf("Scheisse gelaufen mit der Datei...\n");
        return -1;
    }else{
        char msg[BUF];
        fseek(fp, 0L, SEEK_END);
        int filesize = ftell(fp);
        sprintf(msg, "Datei %s geöffnet. Speicheradresse %p, Speichergröße %d Byte\n", filepath, &fp, filesize);
        debugmsg(msg);
        count_lines(fp);
        char *access_log_buffer = malloc(BUF);
        struct access_log_entry_t *access_log_table = malloc(sizeof(struct access_log_entry_t)*BUF);
        if(access_log_buffer==NULL){
            perror("Fuck\n");
            return -1;
        };
        rewind(fp);
        for (int counter = 0; counter < 10 && fgets(access_log_buffer, filesize, fp); counter++) {
            //debugmsg("entered loop");
            char outputline[BUF];
            sprintf(outputline, "LINE %d: %s", counter+1, access_log_buffer);
            fputs(outputline, stdout);
            //debugmsg("endloop");
        }
        free(access_log_buffer);
        free(access_log_table);
        //access_log_buffer = NULL;
        access_log_table = NULL;
    }
    debugmsg("END OF PROGRAM");
    return 0;
}